Package: formods
Type: Package
Title: 'Shiny' Modules for General Tasks
Version: 0.2.2
Authors@R: c(person("John", "Harrold",
                    role    = c("aut", "cre"),
                    email   = "john.m.harrold@gmail.com",
                    comment = c(ORCID="0000-0003-2052-4373")))
Maintainer: John Harrold <john.m.harrold@gmail.com>
Description: 'Shiny' apps can often make use of the same key elements, this package provides modules for common tasks (data upload, wrangling data, figure generation and saving the app state), and also a framework for developing. These modules can react and interact as well as generate code to create reproducible analyses.
License: BSD_2_clause + file LICENSE
BugReports: https://github.com/john-harrold/formods/issues
URL: https://formods.ubiquity.tools/
Encoding: UTF-8
Depends: R (>= 4.2.0)
Imports: cli, digest, dplyr, ggforce, ggplot2, onbrand (>= 1.0.3),
        readr, readxl, rlang, rhandsontable, shiny, shinyAce,
        shinyWidgets, stats, stringr, tools, writexl, yaml, zip
Suggests: clipr, covr, devtools, DT, flextable, ggpubr, gtools, here,
        janitor, knitr, plotly, prompter, rmarkdown, shinybusy,
        shinydashboard, testthat (>= 3.0.0), utils
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2025-10-29 03:55:58 UTC; jmh
Author: John Harrold [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2052-4373>)
Repository: CRAN
Date/Publication: 2025-10-29 06:10:16 UTC
Built: R 4.6.0; ; 2026-04-30 20:12:22 UTC; unix
