# Generated by roxygen2: do not edit by hand

export(ASM_Server)
export(ASM_fetch_code)
export(ASM_fetch_dlfn)
export(ASM_fetch_state)
export(ASM_init_state)
export(ASM_load_state)
export(ASM_mk_preload)
export(ASM_onload)
export(ASM_preload)
export(ASM_read_app_state)
export(ASM_save_state)
export(ASM_test_mksession)
export(DW_Server)
export(DW_add_wrangling_element)
export(DW_append_report)
export(DW_attach_ds)
export(DW_fetch_code)
export(DW_fetch_current_view)
export(DW_fetch_ds)
export(DW_fetch_state)
export(DW_hasds)
export(DW_init_state)
export(DW_mk_preload)
export(DW_new_view)
export(DW_preload)
export(DW_set_current_view)
export(DW_test_mksession)
export(DW_update_checksum)
export(FG_Server)
export(FG_append_report)
export(FG_build)
export(FG_extract_page)
export(FG_fetch_code)
export(FG_fetch_current_fig)
export(FG_fetch_state)
export(FG_init_state)
export(FG_mk_preload)
export(FG_new_fig)
export(FG_preload)
export(FG_set_current_fig)
export(FG_test_mksession)
export(FG_update_checksum)
export(FM_add_ui_tooltip)
export(FM_app_preload)
export(FM_build_comment)
export(FM_fetch_app_code)
export(FM_fetch_app_info)
export(FM_fetch_app_state)
export(FM_fetch_current_mods)
export(FM_fetch_data_format)
export(FM_fetch_deps)
export(FM_fetch_ds)
export(FM_fetch_log_path)
export(FM_fetch_mdl)
export(FM_fetch_mod_state)
export(FM_fetch_user_files_path)
export(FM_generate_report)
export(FM_init_state)
export(FM_le)
export(FM_message)
export(FM_mk_app_preload)
export(FM_mk_error_fig)
export(FM_notify)
export(FM_pause_screen)
export(FM_pretty_sort)
export(FM_proc_include)
export(FM_reset_app)
export(FM_resume_screen)
export(FM_set_app_state)
export(FM_set_mod_state)
export(FM_set_notification)
export(FM_set_ui_msg)
export(FM_tc)
export(UD_Server)
export(UD_attach_ds)
export(UD_ds_read)
export(UD_fetch_code)
export(UD_fetch_ds)
export(UD_fetch_state)
export(UD_init_state)
export(UD_mk_preload)
export(UD_preload)
export(UD_test_mksession)
export(autocast)
export(dw_eval_element)
export(dwrs_builder)
export(fers_builder)
export(fetch_hold)
export(fetch_package_version)
export(formods_check)
export(has_changed)
export(has_updated)
export(icon_link)
export(is_installed)
export(is_shiny)
export(linspace)
export(new_module_template)
export(remove_hold)
export(render_str)
export(run_formods)
export(set_hold)
export(unfactor)
export(use_formods)
import(cli)
import(dplyr)
import(ggplot2)
import(readxl)
import(rhandsontable)
import(shiny)
import(shinyWidgets)
importFrom(digest,digest)
importFrom(ggforce,facet_grid_paginate)
importFrom(ggforce,facet_wrap_paginate)
importFrom(readr,read_csv)
importFrom(rlang,.data)
importFrom(shinyAce,aceEditor)
importFrom(shinyAce,updateAceEditor)
importFrom(stats,setNames)
importFrom(stringr,str_replace_all)
importFrom(tools,file_ext)
importFrom(writexl,write_xlsx)
importFrom(yaml,read_yaml)
importFrom(zip,unzip)
importFrom(zip,zip)
